
From: John McKell <mckellj@iomega.com>

This 2.6.1 patch works by setting gendisk->policy to the correct value
during initialization as the various drivers decide whether or not the disk
is writeable.  This patch persuades "blockdev --getro ..." to correctly
report the read-only state of a newly inserted disk.  This patch applies to
sr.c, sd.c and ide-floppy.c.  ide-cd.c already has this functionality built
into it.

Using an Iomega Zip drive as the test case...

Without the patch, I always see: 

$ sudo blockdev --getro /dev/sda 
0
$

That's only correct for writeable disks though.  Only when the patch
is applied do I see a write-protected disk described correctly:

$ sudo blockdev --getro /dev/sda 
1
$



---

 drivers/ide/ide-floppy.c |    1 +
 drivers/scsi/sd.c        |    2 ++
 drivers/scsi/sr.c        |    8 +++++++-
 3 files changed, 10 insertions(+), 1 deletion(-)

diff -puN drivers/ide/ide-floppy.c~fix-blockdev-getro drivers/ide/ide-floppy.c
--- 25/drivers/ide/ide-floppy.c~fix-blockdev-getro	2004-01-22 00:48:39.000000000 -0800
+++ 25-akpm/drivers/ide/ide-floppy.c	2004-01-22 00:48:39.000000000 -0800
@@ -1317,6 +1317,7 @@ static int idefloppy_get_flexible_disk_p
 	}
 	header = (idefloppy_mode_parameter_header_t *) pc.buffer;
 	floppy->wp = header->wp;
+	set_disk_ro(drive->disk, floppy->wp);
 	page = (idefloppy_flexible_disk_page_t *) (header + 1);
 
 	page->transfer_rate = ntohs(page->transfer_rate);
diff -puN drivers/scsi/sd.c~fix-blockdev-getro drivers/scsi/sd.c
--- 25/drivers/scsi/sd.c~fix-blockdev-getro	2004-01-22 00:48:39.000000000 -0800
+++ 25-akpm/drivers/scsi/sd.c	2004-01-22 00:48:39.000000000 -0800
@@ -1093,6 +1093,7 @@ sd_read_write_protect_flag(struct scsi_d
 	int res;
 	struct scsi_mode_data data;
 
+	set_disk_ro(sdkp->disk, 0);
 	if (sdkp->device->skip_ms_page_3f) {
 		printk(KERN_NOTICE "%s: assuming Write Enabled\n", diskname);
 		return;
@@ -1124,6 +1125,7 @@ sd_read_write_protect_flag(struct scsi_d
 		       "%s: test WP failed, assume Write Enabled\n", diskname);
 	} else {
 		sdkp->write_prot = ((data.device_specific & 0x80) != 0);
+		set_disk_ro(sdkp->disk, sdkp->write_prot);
 		printk(KERN_NOTICE "%s: Write Protect is %s\n", diskname,
 		       sdkp->write_prot ? "on" : "off");
 		printk(KERN_DEBUG "%s: Mode Sense: %02x %02x %02x %02x\n",
diff -puN drivers/scsi/sr.c~fix-blockdev-getro drivers/scsi/sr.c
--- 25/drivers/scsi/sr.c~fix-blockdev-getro	2004-01-22 00:48:39.000000000 -0800
+++ 25-akpm/drivers/scsi/sr.c	2004-01-22 00:49:29.000000000 -0800
@@ -712,6 +712,9 @@ static void get_capabilities(struct scsi
 		""
 	};
 
+	/* Set read only initially */
+	set_disk_ro(cd->disk, 1);
+
 	/* allocate a request for the TEST_UNIT_READY */
 	SRpnt = scsi_allocate_request(cd->device, GFP_KERNEL);
 	if (!SRpnt) {
@@ -825,8 +828,11 @@ static void get_capabilities(struct scsi
 	/*
 	 * if DVD-RAM of MRW-W, we are randomly writeable
 	 */
-	if ((cd->cdi.mask & (CDC_DVD_RAM | CDC_MRW_W)) != (CDC_DVD_RAM | CDC_MRW_W))
+	if ((cd->cdi.mask & (CDC_DVD_RAM | CDC_MRW_W)) !=
+			(CDC_DVD_RAM | CDC_MRW_W)) {
 		cd->device->writeable = 1;
+		set_disk_ro(cd->disk, 0);
+	}
 
 	scsi_release_request(SRpnt);
 	kfree(buffer);

_
